import type { FunctionalComponent, Ref } from 'vue';
import type { HookType } from './constants';
export declare type NullablePartial<T> = {
    [P in keyof T]?: T[P] | undefined | null;
};
export declare type AllItems<T = any> = Item<T> | CustomItem<T> | ItemGroup;
declare type GetComponentOrElementProps<C> = C extends string ? C extends keyof JSX.IntrinsicElements ? JSX.IntrinsicElements[C] : Record<string, unknown> : C extends FunctionalComponent<infer Props> ? Props : C extends new () => {
    $props: infer Props;
} ? Props : Record<string, unknown>;
declare type GetElemenAttrs<T> = T extends string ? T extends keyof JSX.IntrinsicElements ? JSX.IntrinsicElements[T] : Record<string, unknown> : Record<string, unknown>;
export interface ItemGroup<WrapperComponent = unknown, Props = GetComponentOrElementProps<WrapperComponent>> {
    key: string;
    type: symbol;
    wrapperComponentOrTag?: WrapperComponent;
    wrapperProps?: Props;
    items: AllItems[];
}
export interface Item<Meta = unknown, WrapperComponent = unknown, ElementTag = unknown, WrapperProps = GetComponentOrElementProps<WrapperComponent>, ElementAttrs = {
    key?: undefined;
    ref?: Ref<HTMLElement | null | undefined> | ((el: HTMLElement) => void);
} & GetElemenAttrs<ElementTag>> {
    key: string;
    type: symbol;
    wrapperComponentOrTag?: WrapperComponent;
    wrapperProps?: WrapperProps;
    elementTag?: ElementTag;
    elementAttrs?: ElementAttrs;
    disabled: boolean;
    onSelect?: (thisMeta: Meta, thisItem: Item<Meta, WrapperComponent, ElementTag>, thisEl: HTMLElement) => void;
    meta?: Meta;
}
export interface CustomItem<Meta = unknown> {
    key: string;
    type: symbol;
    name: string;
    meta?: Meta;
}
export declare type Hook = (meta: any, item: Item<any>, el: HTMLElement) => void;
export declare type SelectHook = Hook;
export declare type UnfocusHook = Hook;
export declare type FocusHook = (meta: any, item: Item<any>, el: HTMLElement, byPointer: boolean) => void;
export declare type DOMFocusHook = (event: FocusEvent, meta: any, item: Item<any>, el: HTMLElement) => void;
export interface HookFnMap {
    [HookType.DOMFocus]: DOMFocusHook;
    [HookType.Select]: SelectHook;
    [HookType.Focus]: FocusHook;
    [HookType.Unfocus]: UnfocusHook;
}
export declare type CustomItemOptions<Meta = unknown> = Omit<CustomItem<Meta>, 'type'>;
export declare type ItemOptions<Meta = unknown, WrapperComponent = unknown, ElementTag = unknown> = Omit<Item<Meta, WrapperComponent, ElementTag>, 'type' | 'disabled'> & {
    disabled?: boolean;
};
export declare type ItemGroupOptions<WrapperComponent = unknown> = Omit<ItemGroup<WrapperComponent>, 'type'>;
export declare type ItemDefaults = {
    wrapperComponentOrTag: unknown;
    wrapperProps: Record<string, unknown>;
    elementTag: string;
    elementAttrs: Record<string, unknown>;
};
export {};
